'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var dippy = require('@tinkoff/dippy');
var core = require('@tramvai/core');
var tokensHttpClient = require('@tramvai/tokens-http-client');
var tokensCommon = require('@tramvai/tokens-common');
var httpClientFactory = require('./httpClient/httpClientFactory.js');
var papiClientModule = require('./papiClientModule.js');

const createCacheToken = dippy.createToken('httpClient createCache');
const HttpClientModule = /* @__PURE__ */ core.Module({
    imports: [papiClientModule.PapiClientModule],
    providers: [
        core.provide({
            provide: tokensHttpClient.HTTP_CLIENT_FACTORY,
            useFactory: httpClientFactory.httpClientFactory,
            deps: {
                logger: tokensCommon.LOGGER_TOKEN,
                envManager: tokensCommon.ENV_MANAGER_TOKEN,
                appInfo: core.APP_INFO_TOKEN,
                createCache: createCacheToken,
                makeRequestRegistry: 'makeRequestRegistry',
                requestManager: {
                    token: tokensCommon.REQUEST_MANAGER_TOKEN,
                    optional: true,
                },
                headersList: {
                    token: tokensHttpClient.API_CLIENT_PASS_HEADERS,
                    optional: true,
                },
                agent: {
                    token: tokensHttpClient.HTTP_CLIENT_AGENT,
                    optional: true,
                },
                disableCircuitBreaker: {
                    token: tokensHttpClient.DISABLE_CIRCUIT_BREAKER,
                    optional: true,
                },
                defaultOptions: {
                    token: tokensHttpClient.DEFAULT_HTTP_CLIENT_FACTORY_OPTIONS,
                    optional: true,
                },
                defaultInterceptors: {
                    token: tokensHttpClient.DEFAULT_HTTP_CLIENT_INTERCEPTORS,
                    optional: true,
                },
                commandLineExecutionContext: {
                    token: tokensCommon.COMMAND_LINE_EXECUTION_CONTEXT_TOKEN,
                    optional: true,
                },
            },
        }),
        core.provide({
            provide: tokensHttpClient.HTTP_CLIENT,
            useFactory: ({ factory }) => {
                return factory({
                    name: 'http-client',
                    enableCircuitBreaker: false,
                });
            },
            deps: {
                factory: tokensHttpClient.HTTP_CLIENT_FACTORY,
            },
        }),
        core.provide({
            provide: tokensCommon.ENV_USED_TOKEN,
            useValue: [
                { key: 'HTTP_CLIENT_CACHE_DISABLED', optional: true, dehydrate: false },
                { key: 'HTTP_CLIENT_CIRCUIT_BREAKER_DISABLED', optional: true, dehydrate: false },
            ],
        }),
        /**
         * хранилище для экземпляров @tinkoff/request
         *
         * требуется хранить экземпляры в единственном виде,
         * т.к. многие плагины @tinkoff/request после инициализации имеют состояние
         * (cache, circuit breaker), и не будут корректно работать на сервере,
         * если создавать новые экземпляры на Scope.REQUEST
         */
        core.provide({
            provide: 'makeRequestRegistry',
            scope: core.Scope.SINGLETON,
            useFactory: () => new Map(),
        }),
        /**
         * `CREATE_CACHE_TOKEN` имеет проверку, если токен используется провайдером,
         * который имеет Scope.SINGLETON, то инстанс кэша сохраняется в общее хранилище,
         * и доступен для очистки через `/papi/clear-cache`.
         * Scope.REQUEST игнорируется, т.к. это верная утечка памяти,
         * инстансов кэши было бы неограниченное количество.
         *
         * HTTP клиенты создаются со Scope.REQUEST, но инстансы @tinkoff/request
         * (и соответственно кэшей) создаются только один раз, благодаря `makeRequestRegistry`.
         * это гарантирует отсутствие утечек памяти, поэтому мы обходим проверку
         * на Scope.SINGLETON c помощью обертки `createCacheToken`
         */
        core.provide({
            provide: createCacheToken,
            scope: core.Scope.SINGLETON,
            useFactory: ({ createCache }) => {
                return createCache;
            },
            deps: {
                createCache: tokensCommon.CREATE_CACHE_TOKEN,
            },
        }),
        core.provide({
            provide: tokensHttpClient.API_CLIENT_PASS_HEADERS,
            useValue: ['x-request-id'],
        }),
    ],
})(class HttpClientModule {
});

exports.HttpClientModule = HttpClientModule;
