import find from '@tinkoff/utils/array/find';
import flatten from '@tinkoff/utils/array/flatten';
import { BaseHttpClient } from '@tramvai/http-client';
import { createChildContainer } from '@tinkoff/dippy';
import { getPapiParameters } from '@tramvai/papi';
import { FASTIFY_REQUEST, FASTIFY_RESPONSE, PAPI_EXECUTOR } from '@tramvai/tokens-server-private';
import '@tinkoff/utils/function/identity';
import '@tinkoff/utils/object/pick';
import { comparePathWithPattern } from '../utils/comparePathWithPattern.es.js';
import { getPathParams } from '../utils/getPathParams.es.js';

class PapiService extends BaseHttpClient {
    constructor({ papi, di }) {
        super();
        this.papi = flatten(papi || []);
        this.di = di;
    }
    async request({ path, query, body, headers, }) {
        var _a;
        const pathWithLeadingSlash = (path === null || path === void 0 ? void 0 : path.startsWith('/')) ? path : `/${path}`;
        const papiRoute = find((papi) => comparePathWithPattern(pathWithLeadingSlash, getPapiParameters(papi).path), (_a = this.papi) !== null && _a !== void 0 ? _a : []);
        if (!papiRoute) {
            throw new Error(`papi handler '${path}' not found`);
        }
        const req = {
            headers: { ...headers, host: 'localhost' },
            cookies: {},
            query,
            body,
            params: getPathParams(pathWithLeadingSlash, getPapiParameters(papiRoute).path),
        };
        const res = {};
        const childDi = createChildContainer(this.di, [
            {
                provide: FASTIFY_REQUEST,
                useValue: req,
            },
            {
                provide: FASTIFY_RESPONSE,
                useValue: res,
            },
        ]);
        const papiExecutor = childDi.get(PAPI_EXECUTOR);
        const payload = await papiExecutor(papiRoute);
        return { payload, status: 200, headers: {} };
    }
}

export { PapiService };
