'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var find = require('@tinkoff/utils/array/find');
var flatten = require('@tinkoff/utils/array/flatten');
var httpClient = require('@tramvai/http-client');
var dippy = require('@tinkoff/dippy');
var papi = require('@tramvai/papi');
var tokensServerPrivate = require('@tramvai/tokens-server-private');
require('@tinkoff/utils/function/identity');
require('@tinkoff/utils/object/pick');
var comparePathWithPattern = require('../utils/comparePathWithPattern.js');
var getPathParams = require('../utils/getPathParams.js');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var find__default = /*#__PURE__*/_interopDefaultLegacy(find);
var flatten__default = /*#__PURE__*/_interopDefaultLegacy(flatten);

class PapiService extends httpClient.BaseHttpClient {
    constructor({ papi, di }) {
        super();
        this.papi = flatten__default["default"](papi || []);
        this.di = di;
    }
    async request({ path, query, body, headers, }) {
        var _a;
        const pathWithLeadingSlash = (path === null || path === void 0 ? void 0 : path.startsWith('/')) ? path : `/${path}`;
        const papiRoute = find__default["default"]((papi$1) => comparePathWithPattern.comparePathWithPattern(pathWithLeadingSlash, papi.getPapiParameters(papi$1).path), (_a = this.papi) !== null && _a !== void 0 ? _a : []);
        if (!papiRoute) {
            throw new Error(`papi handler '${path}' not found`);
        }
        const req = {
            headers: { ...headers, host: 'localhost' },
            cookies: {},
            query,
            body,
            params: getPathParams.getPathParams(pathWithLeadingSlash, papi.getPapiParameters(papiRoute).path),
        };
        const res = {};
        const childDi = dippy.createChildContainer(this.di, [
            {
                provide: tokensServerPrivate.FASTIFY_REQUEST,
                useValue: req,
            },
            {
                provide: tokensServerPrivate.FASTIFY_RESPONSE,
                useValue: res,
            },
        ]);
        const papiExecutor = childDi.get(tokensServerPrivate.PAPI_EXECUTOR);
        const payload = await papiExecutor(papiRoute);
        return { payload, status: 200, headers: {} };
    }
}

exports.PapiService = PapiService;
